package com.totopia.server.auth.controller;

import com.totopia.server.auth.payload.JwtSigninResponse;
import com.totopia.server.auth.payload.SigninRequest;
import com.totopia.server.auth.payload.SignupRequest;
import com.totopia.server.commons.data.payload.ApiResponse;
import com.totopia.server.config.jwt.JwtTokenProvider;
import com.totopia.server.modules.user.entity.RoleEntity;
import com.totopia.server.modules.user.entity.UserEntity;
import com.totopia.server.modules.user.repository.RoleRepository;
import com.totopia.server.modules.user.repository.UserRepository;
import com.totopia.server.modules.user.type.RoleName;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.servlet.support.ServletUriComponentsBuilder;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.net.URI;
import java.util.Collections;

/**
 * Created by rajeevkumarsingh on 02/08/17.
 */
@Slf4j
@RestController
@RequestMapping("/auth")
public class AuthController {

  @Autowired
  AuthenticationManager authenticationManager;

  @Autowired
  UserRepository userRepository;

  @Autowired
  RoleRepository roleRepository;

  @Autowired
  PasswordEncoder passwordEncoder;

  @Autowired
  JwtTokenProvider tokenProvider;

  @Autowired
  HttpServletRequest request;

  @PostMapping("/signin")
  public ResponseEntity<?> authenticateUser(@Valid @RequestBody SigninRequest signinRequest) {

    String ipAddr = this.getIp();
    log.info("ipAddr = " + ipAddr);

    Authentication authentication = authenticationManager.authenticate(
        new UsernamePasswordAuthenticationToken(signinRequest.getUsername(), signinRequest.getPassword()));

    SecurityContextHolder.getContext().setAuthentication(authentication);

    String jwt = tokenProvider.generateToken(authentication);
    return ResponseEntity.ok(new JwtSigninResponse(jwt));
  }

  @PostMapping("/signup")
  public ResponseEntity<?> registerUser(@Valid @RequestBody SignupRequest signupRequest) throws Exception {
    if (userRepository.existsByUsername(signupRequest.getUsername())) {
      return new ResponseEntity<ApiResponse>(
          ApiResponse.builder().success(false).message("Username is already taken!").build(), HttpStatus.BAD_REQUEST);
    }

    if (userRepository.existsByEmail(signupRequest.getEmail())) {
      return new ResponseEntity<ApiResponse>(
          ApiResponse.builder().success(false).message("Email Address is already use!").build(),
          HttpStatus.BAD_REQUEST);
    }

    // Creating user's account
    UserEntity user = UserEntity.builder().username(signupRequest.getUsername()).email(signupRequest.getEmail())
            .nickname(signupRequest.getNickname()).phone(signupRequest.getPhone()).descriptions(signupRequest.getDescriptions())
        .password(signupRequest.getPassword()).block(false).resetCount(0L).sendEmail(true).build();

    user.setPassword(passwordEncoder.encode(user.getPassword()));

    RoleEntity userRole = roleRepository.findByName(RoleName.ROLE_USER)
        .orElseThrow(() -> new Exception("User Role not set."));

    user.setRoles(Collections.singleton(userRole));

    UserEntity result = userRepository.save(user);


    URI location = ServletUriComponentsBuilder.fromCurrentContextPath().path("/users/{username}")
        .buildAndExpand(result.getUsername()).toUri();

    return ResponseEntity.created(location).body(new ApiResponse(true, "User registered successfully"));
  }

  private String getIp() {
      String ip = request.getHeader("X-Forwarded-For");

      log.info(">>>> X-FORWARDED-FOR : " + ip);

      if (ip == null) {
          ip = request.getHeader("Proxy-Client-IP");
          log.info(">>>> Proxy-Client-IP : " + ip);
      }
      if (ip == null) {
          ip = request.getHeader("WL-Proxy-Client-IP"); // 웹로직
          log.info(">>>> WL-Proxy-Client-IP : " + ip);
      }
      if (ip == null) {
          ip = request.getHeader("HTTP_CLIENT_IP");
          log.info(">>>> HTTP_CLIENT_IP : " + ip);
      }
      if (ip == null) {
          ip = request.getHeader("HTTP_X_FORWARDED_FOR");
          log.info(">>>> HTTP_X_FORWARDED_FOR : " + ip);
      }
      if (ip == null) {
          ip = request.getRemoteAddr();
      }

      log.info(">>>> Result : IP Address : "+ip);

      return ip;
  }
}
